﻿Copyright (c) (2017) Fluke Corporation                      MET/CAL Procedure
=============================================================================
INSTRUMENT:            Sub IEEE-488.2 *IDN? Parse Response
INSTRUMENT:            Sub IEEE-488.2 *IDN? Check Response
DATE:                  2017-10-11 07:42:30
AUTHOR:                Fluke
REVISION:              $Revision: 21267 $
ADJUSTMENT THRESHOLD:  70%
NUMBER OF TESTS:       1
NUMBER OF LINES:       300
=============================================================================
#
#  Procedure Author:
#        DAC
#
#  Compatibility:
#        MET/CAL 8.0 or later
#
#  This procedure is intended for use with MET/CAL® calibration software;
#  the terms and conditions set forth in your MET/CAL license apply to this
#  procedure.
#
#  Due to Fluke's policy of continuously updating our products, this procedure
#  may contain minor differences in methods used and/or specifications to
#  those found in the manual or other documentation. While every effort has
#  been made to ensure that this procedure is accurate, Fluke cannot be held
#  responsible for the consequences of error or omissions found within this
#  procedure.
#
#  The copyright in this procedure is owned by Fluke Precision Measurement
#  Limited.
#
 STEP    FSC    RANGE NOMINAL        TOLERANCE     MOD1        MOD2  3  4 CON

  1.001  JMPL         PARSE_RESPONSE               PSUBI("Parse Response")
  1.002  JMPL         CHECK_RESPONSE               PSUBI("Check Response")
  1.003  DISP         Subprocedure not found!
  1.004  MATH         @Abort = 1
  1.005  END

# =====  Sub IEEE-488.2 *IDN? Parse Response  ===============================
#
# The data is organized as four fields separated by commas.  The four fields
# are defined as follows:
#
#   Field 1 Manufacturer                           required
#   Field 2 Model                                  required
#   Field 3 Serial Number                          ASCII "0" if not available
#   Field 4 Firmware Level or equiv.               ASCII "0" if not available
#
# Note that the ASCII character 0 mentioned above is the character "0",
# decimal value 48 and NOT the "null" character, decimal value 0.
#
# The entire length of the response must be <= 72 characters.  The fields may
# not contain commas, semicolons, control characters (decimal 0-31), or
# DEL (127 decimal).  Otherwise, the fields may contain any ASCII encoded
# character.

  1.006  LABEL        PARSE_RESPONSE
  1.007  MATH         @Manufacturer = FLD(MEM2, 1, ",")
  1.008  MATH         @ModelNum     = FLD(MEM2, 2, ",")
  1.009  MATH         @SerialNum    = FLD(MEM2, 3, ",")
  1.010  MATH         @FirmwareRev  = FLD(MEM2, 4, ",")

# Check for field lengths.
  1.011  MATH         LEN_IDN   = LEN(MEM2)
  1.012  MATH         LEN_Manuf = LEN(@Manufacturer)
  1.013  MATH         LEN_Model = LEN(@ModelNum)
  1.014  MATH         LEN_SN    = LEN(@SerialNum)
  1.015  MATH         LEN_FW    = LEN(@FirmwareRev)

# Validate the response.

  1.016  IF           LEN_IDN > 72
  1.017  MATH         Part1 = SUB(MEM2, 1, 56)
  1.018  MATH         Part2 = SUB(MEM2, 57, LEN_IDN)
  1.019  DISP         The DUT Identification Query returned an invalid
  1.019  DISP         response.  The entire length of the response exceeds
  1.019  DISP         characters.
  1.019  DISP
  1.019  DISP         [V Part1]
  1.019  DISP         [V Part2]
  1.020  MATH         @Abort = 1
  1.021  END
  1.022  ENDIF

  1.023  IF           LEN_Manuf < 1
  1.024  DISP         The DUT Identification Query returned an invalid
  1.024  DISP         response for Manufacturer (Field 1):
  1.024  DISP
  1.024  DISP         [MEM2]
  1.025  MATH         @Abort = 1
  1.026  END
  1.027  ENDIF

  1.028  IF           LEN_Model < 1
  1.029  DISP         The DUT Identification Query returned an invalid
  1.029  DISP         response for Model (Field 2):
  1.029  DISP
  1.029  DISP         [MEM2]
  1.030  MATH         @Abort = 1
  1.031  END
  1.032  ENDIF

  1.033  IF           LEN_SN < 1
  1.034  DISP         The DUT Identification Query returned an invalid
  1.034  DISP         response for Serial Number (Field 3):
  1.034  DISP
  1.034  DISP         [MEM2]
  1.035  MATH         @Abort = 1
  1.036  END
  1.037  ENDIF

  1.038  IF           LEN_FW < 1
  1.039  DISP         The DUT Identification Query returned an invalid
  1.039  DISP         response for Firmware Revision (Field 4):
  1.039  DISP
  1.039  DISP         [MEM2]
  1.040  MATH         @Abort = 1
  1.041  END
  1.042  ENDIF

# Some manufacturers pad the fields with a space.  Example:
#     "Hewlett-Packard, E4401B, US39120213, A.06.00"

# Check for leading spaces.
  1.043  MATH         LS_Manuf = ZCMP(SUB(@Manufacturer, 1, 1), " ")
  1.044  MATH         LS_Model = ZCMP(SUB(@ModelNum,     1, 1), " ")
  1.045  MATH         LS_SN    = ZCMP(SUB(@SerialNum,    1, 1), " ")
  1.046  MATH         LS_FW    = ZCMP(SUB(@FirmwareRev,  1, 1), " ")

# Strip leading whitespace from fields where required.

# Check model field for leading whitespace.
  1.047  IF           LS_Model && (LEN_Model > 1)
  1.048  MATH         @ModelNum = STRIPL(@ModelNum)
  1.049  ENDIF

# Check serial number field for "0" or leading whitespace.
  1.050  IF           ZCMP(@SerialNum, "0") || ZCMP(@SerialNum, " 0")
  1.051  MATH         @SerialNum = "Not available"
  1.052  ELSEIF       LS_SN && (LEN_SN > 1)
  1.053  MATH         @SerialNum = STRIPL(@SerialNum)
  1.054  ENDIF

# Check firmware revision field for "0" or leading whitespace.
  1.055  IF           ZCMP(@FirmwareRev, "0") || ZCMP(@FirmwareRev, " 0")
  1.056  MATH         @FirmwareRev = "Not available"
  1.057  ELSEIF       LS_FW && (LEN_FW > 1)
  1.058  MATH         @FirmwareRev = STRIPL(@FirmwareRev)
  1.059  ENDIF

# Make sure firmware level string is not too big for DISP statement line.
  1.060  MATH         Max = 56 - LEN("Firmware Level:") - 2
  1.061  MATH         LenFirmwareRev = LEN(@FirmwareRev)

  1.062  IF           LenFirmwareRev > Max
  1.063  MATH         Part1 = SUB(@FirmwareRev, 1, Max)
  1.064  MATH         Part2 = SUB(@FirmwareRev, (Max + 1), LenFirmwareRev)
  1.065  DISP         Manufacturer:    [V @Manufacturer]
  1.065  DISP         Model:           [V @ModelNum]
  1.065  DISP         Serial Number:   [V @SerialNum]
  1.065  DISP         Firmware Level:  [V Part1]
  1.065  DISP         [32]                [V Part2]
# Write serial number and firmware revision to results file.
  1.066  RSLT         =
  1.067  RSLT         = Serial Number:   [V @SerialNum]
  1.068  RSLT         = Firmware Level:  [V Part1]
  1.069  RSLT         =[32]                 [V Part2]
  1.070  ELSE
  1.071  DISP         Manufacturer:    [V @Manufacturer]
  1.071  DISP         Model:           [V @ModelNum]
  1.071  DISP         Serial Number:   [V @SerialNum]
  1.071  DISP         Firmware Level:  [V @FirmwareRev]
# Write serial number and firmware revision to results file.
  1.072  RSLT         =
  1.073  RSLT         = Serial Number:   [V @SerialNum]
  1.074  RSLT         = Firmware Level:  [V @FirmwareRev]
  1.075  ENDIF

  1.076  MATH         @Abort = 0
  1.077  END

# =====  Sub IEEE-488.2 *IDN? Check Response  ===============================

# Example usage (IEEE-488):
#
#  1.001  MATH         @DUT_Manufacturer  = "Agilent Technologies"
#  1.002  MATH         @DUT_Manufacturer2 = "Hewlett-Packard"
#  1.003  MATH         @DUT_Model         = "E4443A"
#  1.004  MATH         @DUT_AssetNum      = UUT()
#
#  1.005  IF           NOT(EMPTY(@DUT_AssetNum))
#  1.006  MATH         @DUT_SerialNum = SN(@DUT_AssetNum)
#  1.007  ENDIF
#
#  1.008  CALL         Sub IEEE-488.2 *IDN? IEEE
#  1.009  CALL         Sub IEEE-488.2 *IDN? Check Response
#
#  1.010  IF           @Abort
#  1.011  END
#  1.012  ENDIF
#
# Example usage (RS-232):
#
#  1.001  PORT         [P9600,N,8,1,X][TERM LF][OTERM LF]
#
#  1.002  MATH         @DUT_Manufacturer  = "FLUKE"
#  1.003  MATH         @DUT_Model         = "45"
#  1.004  MATH         @DUT_AssetNum      = UUT()
#
#  1.005  IF           NOT(EMPTY(@DUT_AssetNum))
#  1.006  MATH         @DUT_SerialNum = SN(@DUT_AssetNum)
#  1.007  ENDIF
#
#  1.008  CALL         Sub IEEE-488.2 *IDN? RS-232 using OTERM
#  Read an discard acknowledge response.
#  1.009  PORT         [I!]
#  1.010  CALL         Sub IEEE-488.2 *IDN? Check Response
#
#  1.011  IF           @Abort
#  1.012  END
#  1.013  ENDIF

  1.078  LABEL        CHECK_RESPONSE
# Verify all required global variables are populated.
  1.079  MATH         @Abort = NOT(ISVAR("@Manufacturer"))

  1.080  IF           @Abort
  1.081  DISP         "@Manufacturer" must contain *IDN? response, field 1,
  1.081  DISP         returned from DUT.
  1.082  END
  1.083  ENDIF

  1.084  MATH         @Abort = NOT(ISVAR("@ModelNum"))

  1.085  IF           @Abort
  1.086  DISP         "@ModelNum" must contain *IDN? response, field 2,
  1.086  DISP         returned from DUT.
  1.087  END
  1.088  ENDIF

  1.089  MATH         @Abort = NOT(ISVAR("@DUT_Manufacturer"))

  1.090  IF           @Abort
  1.091  DISP         "@DUT_Manufacturer" must contain actual manufacturer
  1.091  DISP         name expected.
  1.092  END
  1.093  ENDIF

  1.094  MATH         @Abort = NOT(ISVAR("@DUT_Model"))

  1.095  IF           @Abort
  1.096  DISP         "@DUT_Model" must contain actual manufacturer model
  1.096  DISP         number expected.
  1.097  END
  1.098  ENDIF

  1.099  MATH         L[1] = ZCMPI(@ModelNum, @DUT_Model)
  1.100  MATH         L[2] = ZCMPI(@Manufacturer, @DUT_Manufacturer)

  1.101  IF           ISVAR("@DUT_Manufacturer2")
  1.102  MATH         L[3] = ZCMPI(@Manufacturer, @DUT_Manufacturer2)
  1.103  ELSE
  1.104  MATH         L[3] = 1
  1.105  ENDIF

  1.106  MATH         @Abort = NOT(L[1] && (L[2] || L[3]))

  1.107  IF           @Abort

  1.108  IF           ISVAR("@DUT_Manufacturer2")
  1.109  DISP         DUT is a [V @Manufacturer] [V @ModelNum],
  1.109  DISP         not a [V @DUT_Manufacturer] [V @DUT_Model]
  1.109  DISP         or [V @DUT_Manufacturer2] [V @DUT_Model]!
  1.110  ELSE
  1.111  DISP         DUT is a [V @Manufacturer] [V @ModelNum],
  1.111  DISP         not a [V @DUT_Manufacturer] [V @DUT_Model]!
  1.112  ENDIF

  1.113  END
  1.114  ENDIF

# Check the serial number if the calling procedure specified a
# serial number and the DUT returned a serial number.
  1.115  IF           ISVAR("@DUT_SerialNum")

  1.116  IF           NOT(ZCMPI(@SerialNum, "Not available"))
  1.117  MATH         @Abort = NOT(ZCMPI(@SerialNum, @DUT_SerialNum))

  1.118  IF           @Abort
  1.119  DISP         DUT serial number is [V @SerialNum].
  1.119  DISP         Expected serial number is [V @DUT_SerialNum]!
  1.120  ENDIF

  1.121  ENDIF

  1.122  ENDIF        ; ISVAR("@DUT_SerialNum")

  1.123  END
